@@ -44,6 +44,7 @@ demandimport.ignore += ['_imp', '_thread', 'axolotl', 'PIL',
 
 IQ_CALLBACK = {}
 
+CRYPTOGRAPHY_MISSING = 'You are missing Python-Cryptography'
 AXOLOTL_MISSING = 'You are missing Python-Axolotl or use an outdated version'
 PROTOBUF_MISSING = 'OMEMO cant import Google Protobuf, you can find help in ' \
                    'the GitHub Wiki'
@@ -58,6 +59,12 @@ DB_DIR_NEW = configpaths.gajimpaths['MY_DATA']
 
 log = logging.getLogger('gajim.plugin_system.omemo')
 
+try:
+    from .file_decryption import FileDecryption
+except Exception as e:
+    log.exception(e)
+    ERROR_MSG = CRYPTOGRAPHY_MISSING
+
 try:
     prototest = __import__('google.protobuf')
 except Exception as e:
@@ -156,6 +163,12 @@ class OmemoPlugin(GajimPlugin):
         for account in self.plugin.config['DISABLED_ACCOUNTS']:
             self.disabled_accounts.append(account)
 
+        # add aesgcm:// uri scheme to config
+        schemes = gajim.config.get('uri_schemes')
+        if 'aesgcm://' not in schemes.split():
+            schemes += ' aesgcm://'
+            gajim.config.set('uri_schemes', schemes)
+
     def migrate_dbpath(self, account, my_jid):
         old_dbpath = os.path.join(DB_DIR_OLD, 'omemo_' + account + '.db')
         new_dbpath = os.path.join(DB_DIR_NEW, 'omemo_' + my_jid + '.db')
@@ -577,11 +590,11 @@ class OmemoPlugin(GajimPlugin):
             event.msg_iq.delChild('body')
             event.msg_iq.addChild(node=encrypted_node)
 
-            # XEP-xxxx: Explicit Message Encryption
-            if not event.msg_iq.getTag('encrypted', attrs={'xmlns': NS_EME}):
-                eme_node = Node('encrypted', attrs={'xmlns': NS_EME,
-                                                    'name': 'OMEMO',
-                                                    'namespace': NS_OMEMO})
+            # XEP-0380: Explicit Message Encryption
+            if not event.msg_iq.getTag('encryption', attrs={'xmlns': NS_EME}):
+                eme_node = Node('encryption', attrs={'xmlns': NS_EME,
+                                                     'name': 'OMEMO',
+                                                     'namespace': NS_OMEMO})
                 event.msg_iq.addChild(node=eme_node)
 
             # Add Message for devices that dont support OMEMO
@@ -617,6 +630,9 @@ class OmemoPlugin(GajimPlugin):
             -------
             Return if encryption is not activated
         """
+        if event.type_ == 'normal':
+            return False
+
         account = event.account
         if account in self.disabled_accounts:
             return
@@ -687,11 +703,11 @@ class OmemoPlugin(GajimPlugin):
 
             event.msg_iq.addChild(node=encrypted_node)
 
-            # XEP-xxxx: Explicit Message Encryption
-            if not event.msg_iq.getTag('encrypted', attrs={'xmlns': NS_EME}):
-                eme_node = Node('encrypted', attrs={'xmlns': NS_EME,
-                                                    'name': 'OMEMO',
-                                                    'namespace': NS_OMEMO})
+            # XEP-0380: Explicit Message Encryption
+            if not event.msg_iq.getTag('encryption', attrs={'xmlns': NS_EME}):
+                eme_node = Node('encryption', attrs={'xmlns': NS_EME,
+                                                     'name': 'OMEMO',
+                                                     'namespace': NS_OMEMO})
                 event.msg_iq.addChild(node=eme_node)
 
             # Store Hint for MAM
@@ -792,19 +808,25 @@ class OmemoPlugin(GajimPlugin):
         return True
 
     @log_calls('OmemoPlugin')
-    def publish_own_devices_list(self, account):
-        """ Check if the passed event is a device list update and store the new
-            device ids.
+    def publish_own_devices_list(self, account, new=False):
+        """ Get all currently known own active device ids and publish them
 
             Parameters
             ----------
             account : str
                 the account name
+
+            new : bool
+                if True, a devicelist with only one
+                (the current id of this instance) device id is pushed
         """
         state = self.get_omemo_state(account)
-        devices_list = state.own_devices
-        devices_list.append(state.own_device_id)
-        devices_list = list(set(devices_list))
+        if new:
+            devices_list = [state.own_device_id]
+        else:
+            devices_list = state.own_devices
+            devices_list.append(state.own_device_id)
+            devices_list = list(set(devices_list))
         state.set_own_devices(devices_list)
 
         log.debug(account + ' => Publishing own Devices: ' + str(
@@ -823,6 +845,7 @@ class OmemoPlugin(GajimPlugin):
             chat_control : ChatControl
                 Gajim ChatControl object
         """
+        FileDecryption(self).activate(chat_control)
         account = chat_control.contact.account.name
         if account in self.disabled_accounts:
             return
@@ -1075,7 +1098,7 @@ class OmemoPlugin(GajimPlugin):
             devices_list = list(set(unpack_device_list_update(stanza, account)))
             if len(devices_list) == 0:
                 log.error(account + ' => Devicelistquery was NOT successful')
-                self.publish_own_devices_list(account)
+                self.publish_own_devices_list(account, new=True)
                 return False
             contact_jid = stanza.getAttr('from')
             if contact_jid == my_jid:
@@ -1093,7 +1116,7 @@ class OmemoPlugin(GajimPlugin):
                     self.publish_own_devices_list(account)
         else:
             log.error(account + ' => Devicelistquery was NOT successful')
-            self.publish_own_devices_list(account)
+            self.publish_own_devices_list(account, new=True)
 
     @log_calls('OmemoPlugin')
     def clear_device_list(self, account):
