@@ -322,6 +322,8 @@ class Ui(object):
 
     def removeUi(self):
         self.actions_hbox.remove(self.omemobutton)
+        self.chat_control._show_lock_image(False, 'OMEMO', False, True,
+                                           False)
         self.chat_control.prepare_context_menu = \
             self.chat_control.omemo_orig_prepare_context_menu
         self.chat_control.send_message = self.chat_control.orig_send_message
@@ -555,7 +557,7 @@ class OMEMOConfigDialog(GajimPluginConfigDialog):
         for path in paths:
             it = mod.get_iter(path)
             jid, fpr = mod.get(it, 1, 3)
-            fprs.append('%s: %s' % (jid, fpr[4:-5]))
+            fprs.append('%s: %s' % (jid, fpr[31:-12]))
         gtk.Clipboard().set_text('\n'.join(fprs))
         gtk.Clipboard(selection='PRIMARY').set_text('\n'.join(fprs))
 
